use {
    super::{Message, Recipients},
    crate::prelude::Mailbox,
};

#[derive(Debug, Default)]
pub struct Parser {
    id: String,
    from: Option<Mailbox>,
    senders: Vec<Mailbox>,
    recipients: Recipients,
    timestamp: Option<String>,
    title: Option<String>,
    body: String,
}

impl Parser {
    pub fn new(id: &str) -> Self {
        let mut p = Self::default();
        p.id = id.to_string();
        p
    }

    pub fn parse(mut self, content: &str) -> Result<Message, super::Error> {
        let lines = content.lines();
        for l in lines {
            match l {
                s if s.starts_with("<") && self.from.is_none() && self.body.is_empty() => {
                    let s = s.strip_prefix('<').unwrap().trim();
                    let from: Mailbox = s.parse()?;
                    self.from = Some(from);
                }
                s if s.starts_with("<") && self.body.is_empty() => {
                    let sndr = s.strip_prefix('<').unwrap().trim();
                    let from: Mailbox = sndr.parse()?;
                    self.senders.push(from);
                }
                s if s.starts_with(":") && self.body.is_empty() => {
                    self.recipients = s.parse()?;
                }
                s if s.starts_with("@") && self.timestamp.is_none() && self.body.is_empty() => {
                    self.timestamp = Some(s.strip_prefix("@").unwrap().trim().to_string());
                }
                s if s.starts_with("###") && self.title.is_none() => {
                    if let Some(t) = s.strip_prefix("###").map(|x| x.trim().to_string()) {
                        self.title = Some(t);
                    }
                    self.body.push_str(s);
                    self.body.push('\n');
                }
                s if s.starts_with("##") && self.title.is_none() => {
                    if let Some(t) = s.strip_prefix("##").map(|x| x.trim().to_string()) {
                        self.title = Some(t);
                    }
                    self.body.push_str(s);
                    self.body.push('\n');
                }
                s if s.starts_with("#") && self.title.is_none() => {
                    if let Some(t) = s.strip_prefix("#").map(|x| x.trim().to_string()) {
                        self.title = Some(t);
                    }
                    self.body.push_str(s);
                    self.body.push('\n');
                }
                s => {
                    self.body.push_str(s);
                    self.body.push('\n');
                }
            }
        }
        if self.from.is_none() {
            Err(super::Error::EmptySender)
        } else {
            Ok(Message {
                id: self.id,
                from: self.from.unwrap(),
                senders: self.senders,
                recipients: self.recipients.boxes,
                timstamp: self.timestamp,
                title: self.title,
                body: self.body,
            })
        }
    }
}
